package com.villanueva.production.livooks

import android.app.Activity
import android.content.Intent
import android.os.Bundle
import androidx.appcompat.app.AppCompatActivity
import com.firebase.ui.auth.AuthUI
import com.firebase.ui.auth.IdpResponse
import com.google.firebase.auth.FirebaseAuth
import com.villanueva.production.livooks.utils.FirebaseRepo

class FirebaseUIActivity : AppCompatActivity() {

    companion object {
        private const val RC_SIGN_IN = 1
    }

    private lateinit var providers: ArrayList<AuthUI.IdpConfig>

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        // Choose authentication providers
        providers = arrayListOf(
            AuthUI.IdpConfig.EmailBuilder().build(),
            AuthUI.IdpConfig.GoogleBuilder().build(),
            AuthUI.IdpConfig.FacebookBuilder().build(),
            AuthUI.IdpConfig.TwitterBuilder().build()
        )

        //Configure Firebase AuthenticationUI
        showSinginOptions()

    }

    //If the user is sucessfully signed in, add user to database Firestore and go to main screen
    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == RC_SIGN_IN) {
            val response = IdpResponse.fromResultIntent(data)
            if (resultCode == Activity.RESULT_OK) {
                // Successfully signed in
                val user = FirebaseAuth.getInstance().currentUser
                //Add user to Firesotre database
               val  userEmail = user?.email
                if (userEmail != null) {
                    FirebaseRepo().setUser(userEmail)
                }
                //go to main screen
                goToMainActivity()

            } else {
                if (response == null) {
                    finish()
                }

            }
        }
    }

    fun goToMainActivity() {
        val intent = Intent(this, MainActivity::class.java)
        startActivity(intent)
        finish()
    }

    fun showSinginOptions() {
        startActivityForResult(
            AuthUI.getInstance()
                .createSignInIntentBuilder()
                .setAvailableProviders(providers)
                .setLogo(R.drawable.textologolivooks) // Set logo drawable
                .setTheme(R.style.MyStyleSignIn) // Set theme
                .setIsSmartLockEnabled(true)
                /*.setTosAndPrivacyPolicyUrls(
                "https://example.com/terms.html",
                "https://example.com/privacy.html")*/
                .build(), RC_SIGN_IN
        )
    }

}