package com.villanueva.production.livooks

import android.content.Intent
import android.os.Bundle
import android.view.Menu
import android.view.MenuInflater
import android.view.MenuItem
import android.view.View
import android.widget.FrameLayout
import android.widget.SearchView
import androidx.appcompat.app.AppCompatActivity
import androidx.fragment.app.Fragment
import com.google.android.material.bottomnavigation.BottomNavigationView
import com.google.zxing.integration.android.IntentIntegrator
import com.villanueva.production.livooks.fragments.*

const val TAG = "pepe"



class MainActivity : AppCompatActivity() {

    private val MyBooksFragment = MyBooksFragment()
    private val ProgressFragment = ProgressFragment()
    private val DiscoverFragment = DiscoverFragment()
    private val ProfileFragment = ProfileFragment()

    private lateinit var bnNavigationView: BottomNavigationView
    private lateinit var searchView: SearchView
    private lateinit var searchViewTrue: SearchView
    private var fragmentItemId: Int? = null

    private var codeScanner:String? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)

        // Create my own toolbar
        setSupportActionBar(findViewById(R.id.tb_mainToolbar))
        val tbMainToolbar = supportActionBar
        tbMainToolbar?.title = ""

        //Get a support ActionBar corresponding to this toolbar and enable the Up button
        tbMainToolbar?.setDisplayHomeAsUpEnabled(true)

        //Show Fragment default screen my books
        replaceFragment(MyBooksFragment)

        //Configuration navigation toolbar each item call a fragment
        bnNavigationView = findViewById<BottomNavigationView>(R.id.bn_navigationView)

        bnNavigationView.setOnNavigationItemSelectedListener {
            fragmentItemId = it.itemId
            when (it.itemId) {
                R.id.it_myBooks -> {
                    replaceFragment(MyBooksFragment)
                    tbMainToolbar?.show()
                    searchView.visibility = View.VISIBLE
                }

                R.id.it_progress -> replaceFragment(ProgressFragment)
                R.id.it_discover -> {
                    replaceFragment(DiscoverFragment)
                    tbMainToolbar?.show()
                    searchView.visibility = View.VISIBLE
                }
                R.id.it_profile -> replaceFragment(ProfileFragment)

            }
            true
        }

        // Create my own SearchView, when we click in the searchview we will go to Search Fragment
        searchView = findViewById<View>(R.id.sv_searchview) as SearchView
        searchViewTrue = findViewById<View>(R.id.sv_searchviewTrue) as SearchView

        //configuration searchview
        searchView.setOnQueryTextFocusChangeListener { _, _ ->
            bnNavigationView.visibility = View.GONE
            //bnNavigationView.isInvisible = true
            searchView.visibility = View.GONE
            searchViewTrue.visibility = View.VISIBLE
            searchViewTrue.requestFocus()
            searchViewTrue.setQuery("", false)
            //show the keyboard
            searchViewTrue.isIconified = false

            val transaction = supportFragmentManager.beginTransaction()
            transaction.replace(R.id.fc_fragmentContainer, SearchFragment())
            transaction.commit()
        }
        searchView.setOnClickListener {
            searchView.isIconified = false
        }
    }


    //Function for replace fragment depending of item in navigation bar
    private fun replaceFragment(fragment: Fragment) {
        val transaction = supportFragmentManager.beginTransaction()
        transaction.replace(R.id.fc_fragmentContainer, fragment)
        transaction.commit()
    }

    //Create Menu Toolbar
    override fun onCreateOptionsMenu(menu: Menu): Boolean {
        val inflater: MenuInflater = menuInflater
        inflater.inflate(R.menu.menu_toolbar, menu)
        return true
    }
    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        // Handle item selection
        return when (item.itemId) {

            R.id.it_photoBarcode -> {
                // Open camera for get bar code scanning it
                 val integrator = IntentIntegrator(this)
                //integrator.setDesiredBarcodeFormats(IntentIntegrator.ONE_D_CODE_TYPES)
                //integrator.setPrompt("Scan a barcode")
                //integrator.setCameraId(0) // Use a specific camera of the device
                integrator.setBeepEnabled(true)
                integrator.setBarcodeImageEnabled(true)
                integrator.setOrientationLocked(false)
                integrator.initiateScan()
                true
            }

            //Configuration up button in toolbar
            android.R.id.home -> {
                //Go to fragment from where we were previously
                previousFragment()
                true
            }
            else -> super.onOptionsItemSelected(item)
        }
    }

    //function to select the previous fragment
    private fun previousFragment() {
        when (fragmentItemId) {
            R.id.it_myBooks -> replaceFragment(MyBooksFragment)
            R.id.it_progress -> replaceFragment(ProgressFragment)
            R.id.it_discover -> replaceFragment(DiscoverFragment)
            R.id.it_profile -> replaceFragment(ProfileFragment)
            else -> replaceFragment(MyBooksFragment)
        }
        searchView.visibility = View.VISIBLE
        searchViewTrue.visibility = View.GONE
        bnNavigationView.visibility = View.VISIBLE

    }

    //Get result from bar code scanning
    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        val result = IntentIntegrator.parseActivityResult(requestCode, resultCode, data)
        if (result != null) {
            if (result.contents == null) {
                //Toast.makeText(this, "Cancelled", Toast.LENGTH_LONG).show()
            } else {
                codeScanner = result.contents
            }
        } else {
            super.onActivityResult(requestCode, resultCode, data)
        }
    }

    override fun onResume() {
        super.onResume()
        //Go Search fragment when scan a barcode sucessfully
        if (codeScanner!=null){
            val bundle = Bundle()
            bundle.putString("key_codeScanner", codeScanner)
            val searchFragment = SearchFragment()
            searchFragment.arguments = bundle
            replaceFragment(searchFragment)
            bnNavigationView.visibility = View.GONE
            searchView.visibility = View.GONE
            searchViewTrue.visibility = View.VISIBLE
            searchViewTrue.setQuery("", false)
            codeScanner=null
        }

    }

    //Configuration customized back pressed button
    override fun onBackPressed() {
        //val count = supportFragmentManager.backStackEntryCount
        if ((findViewById<FrameLayout>(R.id.rv_listBooks) != null)) {
            super.onBackPressed()
            val intent = Intent(this, MainActivity::class.java)
            startActivity(intent)
            finish()

        } else if ((findViewById<FrameLayout>(R.id.ctb_booklistDetails) != null)) {
            super.onBackPressed()
            searchViewTrue.visibility = View.VISIBLE
            supportActionBar?.show()

        } else if ((findViewById<FrameLayout>(R.id.cl_fragmentCategory) != null)) {
            super.onBackPressed()
            searchView.visibility = View.VISIBLE
            supportActionBar?.show()

        } else if ((findViewById<FrameLayout>(R.id.ctb_myBooksDetails) != null)) {
            super.onBackPressed()
            bnNavigationView.visibility = View.VISIBLE

        } else {
            //super.onBackPressed()
            finish()
        }

    }

}