package com.villanueva.production.livooks.adapters

import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import androidx.recyclerview.widget.RecyclerView
import com.squareup.picasso.Picasso
import com.villanueva.production.livooks.R
import com.villanueva.production.livooks.interfaces.ClickListenerSearch
import com.villanueva.production.livooks.model.modeAPI.BookModelClass

class SearchRecyclerViewAdapter(private val items: ArrayList<BookModelClass>?, var listener: ClickListenerSearch) :
    RecyclerView.Adapter<SearchRecyclerViewAdapter.ViewHolder>() {

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
        val view = LayoutInflater.from(parent.context)
            .inflate(R.layout.template_search_list, parent, false)
        val viewHolder = ViewHolder(view, listener)
        return viewHolder
    }

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {
        val booksSearch = items?.get(position)
        holder.tvTitleSearch?.text = booksSearch?.volumeInfo?.title
        holder.tvAuthorSearch?.text = booksSearch?.volumeInfo?.authors?.get(0)

        val imageUrl = booksSearch?.volumeInfo?.imageLinks?.smallThumbnail

        if(booksSearch?.volumeInfo?.imageLinks?.smallThumbnail == null){
            Picasso.get().load(R.drawable.cover_not_avalaible).into(holder.ivBooksSearch)
        }else{
            Picasso.get()
                .load(imageUrl)
                .into(holder.ivBooksSearch)
        }



    }

    override fun getItemCount(): Int {
        return items?.size!!
    }

    inner class ViewHolder(view: View, listener:ClickListenerSearch) : RecyclerView.ViewHolder(view),View.OnClickListener {
        var tvTitleSearch: TextView? = null
        var tvAuthorSearch: TextView? = null
        var ivBooksSearch: ImageView? = null

        //declare the click listener of our interface (this is for the when to the button on element)
        var listener : ClickListenerSearch? = null

        init {
            tvTitleSearch = view.findViewById(R.id.tv_titleSearch)
            tvAuthorSearch = view.findViewById(R.id.tv_authorSearch)
            ivBooksSearch = view.findViewById(R.id.iv_bookSearch)

            //initialize the listener
            this.listener = listener

            //When it reaches the viewHolder the listener of each view will be activated
            view.setOnClickListener(this)

        }

        override fun onClick(v: View?) {
            //When this onclicK event is executed, I will implement the interface that I have in my
            //clicklistener file through my listener object and I will pass it my view and the current element position
            listener?.onClickSearch(v!!,adapterPosition)
        }

    }

}