package com.villanueva.production.livooks.fragments

import android.content.Context
import android.graphics.Color
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.*
import androidx.appcompat.app.AppCompatActivity
import androidx.appcompat.view.ContextThemeWrapper
import androidx.fragment.app.Fragment
import com.google.android.material.appbar.CollapsingToolbarLayout
import com.google.android.material.snackbar.Snackbar
import com.squareup.picasso.Picasso
import com.villanueva.production.livooks.R
import com.villanueva.production.livooks.utils.FirebaseRepo
import com.villanueva.production.livooks.utils.ShareBook
import java.lang.reflect.Field
import java.lang.reflect.Method


class BookSearchDetailsFragment : Fragment() {

    private lateinit var tvAuthorSearchBook :TextView
    private lateinit var ivSearchBookDetails :ImageView
    private lateinit var tbBookSearchDetails :androidx.appcompat.widget.Toolbar
    private lateinit var tvDescriptionSearch :TextView
    private lateinit var rbSearchBook :RatingBar
    private lateinit var tvCategory1 :TextView

    private lateinit var btAddBooks  :Button

    private lateinit var viewBookSearchDetailsFragment:View

    var title : String? = null
    var authorString : String? = null
    var image : String? = null

    private lateinit var statusBook :String

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        //Bring the data from another fragment
        parentFragmentManager.setFragmentResultListener("key", this,
            { key, bundle ->
                // We use a String here, but any type that can be put in a Bundle is supported
                title = bundle.getString("titleKey")
                val author = bundle.getStringArrayList("authorKey")
                image = bundle.getString("imageKey")
                val description = bundle.getString("descriptionKey")
                val rating = bundle.getFloat("ratingKey")
                val categories = bundle.getStringArrayList("categoriesKey")

                tbBookSearchDetails.title = title
                tvAuthorSearchBook.text = author?.get(0) ?: ""
                authorString = author?.get(0) ?: ""

                if (image == null) {
                    Picasso.get()
                        .load(R.drawable.cover_not_avalaible)
                        .into(ivSearchBookDetails)
                } else {
                    Picasso.get()
                        .load(image)
                        .into(ivSearchBookDetails)
                }

                tvDescriptionSearch.text = description
                rbSearchBook.rating = rating

                tvCategory1.text = categories?.get(0) ?: ""
                if (tvCategory1.text.isEmpty()) {
                    tvCategory1.visibility = View.GONE
                }

            })
        //Change toolbar
        val actionBar = activity as AppCompatActivity?
        actionBar?.findViewById<SearchView>(R.id.sv_searchviewTrue)?.visibility = View.GONE
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {

        // Inflate the layout for this fragment
        viewBookSearchDetailsFragment = inflater.inflate(R.layout.fragment_book_search_details,
            container,
            false
        )

        tvAuthorSearchBook = viewBookSearchDetailsFragment.findViewById(R.id.tv_authorSearchBook)
        ivSearchBookDetails = viewBookSearchDetailsFragment.findViewById(R.id.iv_searchBookDetails)
        tvDescriptionSearch = viewBookSearchDetailsFragment.findViewById(R.id.tv_descriptionSearch)
        rbSearchBook = viewBookSearchDetailsFragment.findViewById(R.id.rb_bookSearch)
        tvCategory1 = viewBookSearchDetailsFragment.findViewById(R.id.tv_category1)

        //custom toolbar
        tbBookSearchDetails = viewBookSearchDetailsFragment.findViewById(R.id.tb_bookSearchDetails)
        val mainActivity = activity as AppCompatActivity?
        mainActivity?.supportActionBar?.hide()

        tbBookSearchDetails.inflateMenu(R.menu.menu_book_search_details)

        tbBookSearchDetails.setNavigationIcon(R.drawable.ic_toolbar_arrow_back_24)
        tbBookSearchDetails.setNavigationOnClickListener {
            fragmentManager?.popBackStack()
            (activity as AppCompatActivity?)!!.supportActionBar?.show()
            val searchView = mainActivity?.findViewById<SearchView>(R.id.sv_searchviewTrue)
            searchView?.visibility = View.VISIBLE
            searchView?.clearFocus()
        }

        //Change color Collapsing toolbar
        val ctbBookListDetails = viewBookSearchDetailsFragment
            .findViewById<CollapsingToolbarLayout>(R.id.ctb_booklistDetails)
        ctbBookListDetails.setExpandedTitleColor(Color.parseColor("#ffffff"))
        ctbBookListDetails.setCollapsedTitleTextColor(Color.parseColor("#ffffff"))

        //Menu Item Toolbar
        tbBookSearchDetails.setOnMenuItemClickListener {
            when (it.itemId) {
                R.id.it_share -> {
                    val shareBook = ShareBook()
                    shareBook.share(this.requireContext(), ivSearchBookDetails, title!!)

                    true
                }

                else -> {
                    super.onOptionsItemSelected(it)
                }
            }
        }

        //Button add book to differents categories by Popup
        btAddBooks = viewBookSearchDetailsFragment.findViewById<Button>(R.id.bt_addBooks)
        btAddBooks.setOnClickListener {
            popupAddBook()
        }

        return viewBookSearchDetailsFragment
    }


    private fun popupAddBook() {
        val menuItemView = viewBookSearchDetailsFragment.findViewById<View>(R.id.bt_addBooks)

        //Put style to popup
        val wrapper: Context = ContextThemeWrapper(this.context, R.style.CustomPopup)
        val popupMenu = PopupMenu(wrapper, menuItemView)

        //For we can put icons with text
        try {
            val fields: Array<Field> = popupMenu.javaClass.declaredFields
            for (field in fields) {
                if ("mPopup" == field.name) {
                    field.isAccessible = true
                    val menuPopupHelper: Any? = field[popupMenu]
                    val classPopupHelper:Class<*> = Class.forName(menuPopupHelper?.javaClass?.name!!)
                    val setForceIcons: Method = classPopupHelper.getMethod(
                        "setForceShowIcon",
                        Boolean::class.javaPrimitiveType
                    )
                    setForceIcons.invoke(menuPopupHelper, true)
                    break
                }
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }

        //continuos con el code as always
        popupMenu.menuInflater.inflate(R.menu.context_menu_addbooks, popupMenu.menu)
        popupMenu.setOnMenuItemClickListener { item ->
            when (item.itemId) {
                //Add book to Reading status category
                R.id.it_reading -> {
                    statusBook = "Reading"
                    //FirebaseRepo().setBookDataReading(statusBook, title, authorString, image, "", "",0F, arrayListOf(""))
                    FirebaseRepo().setBookData(statusBook, title, authorString, image, "", "",0F,
                        null)
                    Snackbar.make(
                        viewBookSearchDetailsFragment,
                        "Libro añadido a leyendo",
                        Snackbar.LENGTH_SHORT
                    ).show()
                }

                //Add book to Read status category
                R.id.it_read -> {
                    statusBook = "Read"
                    FirebaseRepo().setBookData(statusBook, title, authorString, image, "", "",0F,
                        null)
                    Snackbar.make(
                        viewBookSearchDetailsFragment,
                        "Libro añadido a leídos",
                        Snackbar.LENGTH_SHORT
                    ).show()
                }

                //Add book to Pending status category
                R.id.it_pending -> {
                    statusBook = "Pending"
                    FirebaseRepo().setBookData(statusBook, title, authorString, image, "","",0f,
                        null)
                    Snackbar.make(
                        viewBookSearchDetailsFragment,
                        "Libro añadido a pendientes",
                        Snackbar.LENGTH_SHORT
                    ).show()
                }

                //Add book to Wished status category
                R.id.it_wished -> {
                    statusBook = "Wished"
                    FirebaseRepo().setBookData(statusBook, title, authorString, image, "","",0f,
                        null)
                    Snackbar.make(
                        viewBookSearchDetailsFragment,
                        "Libro añadido a lista deseados",
                        Snackbar.LENGTH_SHORT
                    ).show()
                }

                else -> {
                }
            }
            true
        }

        popupMenu.show()
    }

}