package com.villanueva.production.livooks.fragments

import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import com.google.android.material.bottomsheet.BottomSheetDialogFragment
import com.google.android.material.snackbar.Snackbar
import com.squareup.picasso.Picasso
import com.villanueva.production.livooks.R
import com.villanueva.production.livooks.TAG
import com.villanueva.production.livooks.utils.FirebaseRepo
import com.villanueva.production.livooks.utils.ShareBook


class BottomsheetFragment : BottomSheetDialogFragment() {

    private lateinit var ivBottomSheet : ImageView

    var title : String? = ""
    var author : String? = ""
    var image : String? = ""
    var statusBook : String? = ""
    var startDate : String? = ""
    var endDate : String? = ""
    var ratingBook : Float? = 0F
    var quoteListStrings :ArrayList<String>? = arrayListOf("")


    //private lateinit var viewBottomSheetDialogFragment:View
    private lateinit var btLayoutDetailsMovetoReading :View
    private lateinit var btLayoutDetailsMovetoRead :View
    private lateinit var btLayoutDetailsMovetoPending :View
    private lateinit var btLayoutDetailsMovetoWhised :View

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        //change style for doing BottomSheetfragment background transparent
        setStyle(STYLE_NORMAL, R.style.BottomSheetDialogTheme)

       parentFragmentManager.setFragmentResultListener("book_key", this,
           { key, bundle ->
               // We use a String here, but any type that can be put in a Bundle is supported
               title = bundle.getString("book_title_key")
               author = bundle.getString("book_author_key")
               image = bundle.getString("book_image_key")
               statusBook = bundle.getString("book_status_key")
               startDate = bundle.getString("book_start_date_key")
               endDate = bundle.getString("book_end_date_key")
               ratingBook = bundle.getFloat("book_rating_key")

               quoteListStrings = bundle.getStringArrayList("book_quote_key")

               Log.d(TAG, "1")
//               Log.d(TAG, quoteListStrings!![0])

               if (image == null) {
                   Picasso.get()
                       .load(R.drawable.cover_not_avalaible)
                       .into(ivBottomSheet)
               } else {
                   Picasso.get()
                       .load(image)
                       .into(ivBottomSheet)
               }

               //Depending of status book, eliminate the option not necesary in the bottomSheet
              when(statusBook){
                  "Reading" -> btLayoutDetailsMovetoReading.visibility = View.GONE
                  "Read" -> btLayoutDetailsMovetoRead.visibility = View.GONE
                  "Pending" -> btLayoutDetailsMovetoPending.visibility = View.GONE
                  "Wished" -> btLayoutDetailsMovetoWhised.visibility = View.GONE
              }


           })


    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        // Inflate the layout for this fragment

        val viewBottomSheetDialogFragment = inflater.inflate(
            R.layout.fragment_bottomsheet,
            container,
            false
        )

        //Move to reading
        btLayoutDetailsMovetoReading = viewBottomSheetDialogFragment.findViewById<View>(R.id.layoutDetails_btMoveReading)
        btLayoutDetailsMovetoReading.setOnClickListener {
            FirebaseRepo().deleteBook(title,statusBook!! )
            FirebaseRepo().setBookData("Reading", title, author, image, startDate, endDate,ratingBook,quoteListStrings)
            //firebaseRepo().setQuote(statusBook!!, title,etmQuote.text.toString())

            //Snackbar.make(viewBottomSheetDialogFragment, "Libro movido a leyendo", Snackbar.LENGTH_SHORT).show()
            //Toast.makeText(context, "Libro movido a leuendo", Toast.LENGTH_SHORT).show()
            if (requireActivity().findViewById<View>(R.id.cl_bookListDetails) != null) {
                Snackbar.make(requireActivity().findViewById(R.id.cl_bookListDetails), "Libro movido a leyendo", Snackbar.LENGTH_LONG).show()
            }else{
                Snackbar.make(requireActivity().findViewById(R.id.cl_fragmentCategory), "Libro movido a leyendo", Snackbar.LENGTH_LONG).show()
            }
            //close bottom sheet dialog
            dismiss()
        }

        //Move to read
        btLayoutDetailsMovetoRead = viewBottomSheetDialogFragment.findViewById<View>(R.id.layoutDetails_btMoveRead)
        btLayoutDetailsMovetoRead.setOnClickListener {
            FirebaseRepo().deleteBook(title,statusBook!! )
            FirebaseRepo().setBookData("Read", title, author, image, startDate, endDate,ratingBook,quoteListStrings)
            if (requireActivity().findViewById<View>(R.id.cl_bookListDetails) != null) {
                Snackbar.make(requireActivity().findViewById(R.id.cl_bookListDetails), "Libro movido a leídos", Snackbar.LENGTH_LONG).show()
            }else{
                Snackbar.make(requireActivity().findViewById(R.id.cl_fragmentCategory), "Libro movido a leídos", Snackbar.LENGTH_LONG).show()
            }
            //close bottom sheet dialog
            dismiss()
        }

        //Move to pending
        btLayoutDetailsMovetoPending = viewBottomSheetDialogFragment.findViewById<View>(R.id.layoutDetails_btMovePending)
        btLayoutDetailsMovetoPending.setOnClickListener {
            FirebaseRepo().deleteBook(title,statusBook!! )
            FirebaseRepo().setBookData("Pending", title, author, image, startDate, endDate,ratingBook,quoteListStrings)
            if (requireActivity().findViewById<View>(R.id.cl_bookListDetails) != null) {
                Snackbar.make(requireActivity().findViewById(R.id.cl_bookListDetails), "Libro movido a pendientes", Snackbar.LENGTH_LONG).show()
            }else{
                Snackbar.make(requireActivity().findViewById(R.id.cl_fragmentCategory), "Libro movido a pendientes", Snackbar.LENGTH_LONG).show()
            }
            //close bottom sheet dialog
            dismiss()
        }

        //Move to whised
        btLayoutDetailsMovetoWhised = viewBottomSheetDialogFragment.findViewById<View>(R.id.layoutDetails_btMoveWhised)
        btLayoutDetailsMovetoWhised.setOnClickListener {
            //FirebaseRepo().deleteBook(title,statusBook!! )
            FirebaseRepo().setBookData("Wished", title, author, image, startDate,endDate,ratingBook,quoteListStrings)
            if (requireActivity().findViewById<View>(R.id.cl_bookListDetails) != null) {
                Snackbar.make(requireActivity().findViewById(R.id.cl_bookListDetails), "Libro añadido a lista de deseados", Snackbar.LENGTH_LONG).show()
            }else{
                Snackbar.make(requireActivity().findViewById(R.id.cl_fragmentCategory), "Libro añadido a lista de deseados", Snackbar.LENGTH_LONG).show()
            }
            //close bottom sheet dialog
            dismiss()
        }

        //Share book
        ivBottomSheet = viewBottomSheetDialogFragment.findViewById<ImageView>(R.id.iv_botomsheet)
        val btLayoutDetailsShare = viewBottomSheetDialogFragment.findViewById<View>(R.id.layoutDetails_btShare)
        btLayoutDetailsShare.setOnClickListener {
            val shareBook = ShareBook()
            shareBook.share(this.requireContext(), ivBottomSheet, title!!)
        }

        //Delete book
        val btLayoutDetailsDelete = viewBottomSheetDialogFragment.findViewById<View>(R.id.layoutDetails_btDelete)
        btLayoutDetailsDelete.setOnClickListener {
            FirebaseRepo().deleteBook(title,statusBook!! )
            //Snackbar.make(viewBottomSheetDialogFragment, "Libro eliminado", Snackbar.LENGTH_SHORT).show()
            //close bottom sheet dialog
            dismiss()
            //close fragment
            if (requireActivity().findViewById<View>(R.id.cl_bookListDetails) != null) {
                activity?.onBackPressed()
            }

        }

        return viewBottomSheetDialogFragment
    }

}