package com.villanueva.production.livooks.fragments

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ProgressBar
import android.widget.SearchView
import androidx.appcompat.app.AppCompatActivity
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.google.firebase.auth.FirebaseAuth
import com.google.firebase.firestore.ktx.firestore
import com.google.firebase.firestore.ktx.toObject
import com.google.firebase.ktx.Firebase
import com.villanueva.production.livooks.R
import com.villanueva.production.livooks.adapters.ReadingRecyclerViewAdapter
import com.villanueva.production.livooks.interfaces.ClickListenerSearch
import com.villanueva.production.livooks.model.BookFirebaseModel


class CategoryFragment : Fragment() {

    private val db = Firebase.firestore
    private var mAuth: FirebaseAuth = FirebaseAuth.getInstance()

    private lateinit var rvReading: RecyclerView
    private var adapter: ReadingRecyclerViewAdapter? = null

    private lateinit var pbBookReading: ProgressBar
    private val bookList = ArrayList<BookFirebaseModel>()

    private lateinit var viewReadingFragment: View
    private var statusBook: String? = ""
    private lateinit var tbFragmentReading: androidx.appcompat.widget.Toolbar

    private var mainActivity : AppCompatActivity? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        //Bring the data from another fragment
        parentFragmentManager.setFragmentResultListener("statusKey", this,
            { key, bundle ->
                statusBook = bundle.getString("statusBookKey")

                //Depending of status book, put title to toolbar
                when (statusBook) {
                    "Reading" -> tbFragmentReading.title = getString(R.string.ReadingCategory)
                    "Read" -> tbFragmentReading.title = getString(R.string.ReadCategory)
                    "Pending" -> tbFragmentReading.title = getString(R.string.PendingCategory)
                    "Wished" -> tbFragmentReading.title = getString(R.string.WishedCategory)
                }

                pbBookReading.visibility = View.VISIBLE
                //Charge book list of database fiestore
                getAllBooksReading(statusBook!!)
            })

        //Change toolbar
        mainActivity = activity as AppCompatActivity?
        mainActivity?.supportActionBar?.hide()
        mainActivity?.findViewById<SearchView>(R.id.sv_searchview)?.visibility = View.GONE

    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {

        // Inflate the layout for this fragment
        viewReadingFragment = inflater.inflate(R.layout.fragment_category, container, false)

        //Custom toolbar
        tbFragmentReading = viewReadingFragment.findViewById(R.id.tb_fragmentCategory)
        tbFragmentReading.inflateMenu(R.menu.menu_category_books)

        tbFragmentReading.setNavigationIcon(R.drawable.ic_toolbar_arrow_back_24)
        tbFragmentReading.setNavigationOnClickListener {
            fragmentManager?.popBackStack()
            mainActivity?.supportActionBar?.show()
            val searchView = mainActivity?.findViewById<SearchView>(R.id.sv_searchview)
            searchView?.visibility = View.VISIBLE
            tbFragmentReading.visibility = View.GONE

        }

        //Menu para la toolbar
        tbFragmentReading.setOnMenuItemClickListener {
            when (it.itemId) {
                R.id.it_order -> {
                    //It will be implemented later

                    true
                }
                else -> {
                    super.onOptionsItemSelected(it)
                }
            }
        }

        //Depending of status book, put title to toolbar
        when (statusBook) {
            "Reading" -> tbFragmentReading.title = getString(R.string.ReadingCategory)
            "Read" -> tbFragmentReading.title = getString(R.string.ReadCategory)
            "Pending" -> tbFragmentReading.title = getString(R.string.PendingCategory)
            "Wished" -> tbFragmentReading.title = getString(R.string.WishedCategory)
        }

        //Progress bar
        pbBookReading = viewReadingFragment.findViewById<ProgressBar>(R.id.pb_bookCategory)

        //Recycker View
        rvReading = viewReadingFragment.findViewById<RecyclerView>(R.id.rv_category)
        rvReading.setHasFixedSize(true)
        rvReading.layoutManager = LinearLayoutManager(context)

        setupRecyclerView(rvReading)

        return viewReadingFragment
    }

    private fun setupRecyclerView(recyclerView: RecyclerView) {
        adapter = ReadingRecyclerViewAdapter(bookList, object : ClickListenerSearch {
            override fun onClickSearch(view: View, index: Int) {

                val transaction = activity?.supportFragmentManager?.beginTransaction()
                transaction?.replace(R.id.fc_fragmentContainer, MyBookDetailsFragment(), "main")
                    ?.addToBackStack("main")
                transaction?.commit()

                //pass the data from one fragment to another
                val bundle = Bundle()
                bundle.putString("titleKey", bookList[index].title)
                bundle.putString("authorKey", bookList[index].author)
                bundle.putString("imageKey", bookList[index].image)
                bundle.putString("statusKey", statusBook)

                parentFragmentManager.setFragmentResult("key", bundle)
            }

            override fun onClickMenu(view: View, index: Int) {
                val bottomSheetFragment = BottomsheetFragment()
                val bundle = Bundle()
                bundle.putString("book_title_key", bookList[index].title)
                bundle.putString("book_author_key", bookList[index].author)
                bundle.putString("book_image_key", bookList[index].image)
                bundle.putString("book_status_key", statusBook)

                parentFragmentManager.setFragmentResult("book_key", bundle)
                bottomSheetFragment.show(parentFragmentManager, "BotoomSheetDialog")
            }

        })
        recyclerView.adapter = adapter

    }

    //Function for show all books reading and listen for update changes
    private fun getAllBooksReading(statusBook: String) {
        db.collection("Users")
            .document(mAuth.currentUser?.email.toString())
            .collection(statusBook)
            .addSnapshotListener { snapshot, firestoreError ->
                if (firestoreError != null) {
                    return@addSnapshotListener
                }
                bookList.clear()
                for (document in snapshot!!) {
                    val book = document.toObject<BookFirebaseModel>()
                    bookList.add(book)
                }
                adapter?.setItems(bookList)
                pbBookReading.visibility = View.GONE

            }
    }

}