package com.villanueva.production.livooks.fragments

import android.app.AlertDialog
import android.app.DatePickerDialog
import android.graphics.Color
import android.graphics.drawable.ColorDrawable
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.*
import androidx.appcompat.app.AppCompatActivity
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.google.android.gms.tasks.Task
import com.google.android.material.appbar.CollapsingToolbarLayout
import com.google.android.material.bottomnavigation.BottomNavigationView
import com.google.android.material.snackbar.Snackbar
import com.google.firebase.auth.FirebaseAuth
import com.google.firebase.firestore.DocumentSnapshot
import com.google.firebase.firestore.ktx.firestore
import com.google.firebase.ktx.Firebase
import com.squareup.picasso.Picasso
import com.villanueva.production.livooks.R
import com.villanueva.production.livooks.TAG
import com.villanueva.production.livooks.adapters.QuotesRecyclerViewAdapter
import com.villanueva.production.livooks.interfaces.ClickListenerQuote
import com.villanueva.production.livooks.utils.FirebaseRepo
import java.lang.reflect.Field
import java.lang.reflect.Method
import java.util.*
import kotlin.collections.ArrayList

class MyBookDetailsFragment : Fragment() {

    private val db = Firebase.firestore
    private var mAuth: FirebaseAuth = FirebaseAuth.getInstance()

    private var firebaseRepo = FirebaseRepo()

    private lateinit var tvAuthorMyBookDetails: TextView
    private lateinit var ivMyBookDetails: ImageView
    private lateinit var tvDateStartBook: TextView
    private lateinit var tvDateEndBook: TextView
    private lateinit var rbMyBooksDetails: RatingBar
    private lateinit var tbMyBookDetails: androidx.appcompat.widget.Toolbar

    private var title: String? = ""
    private var author: String? = ""
    private var image: String? = ""
    private var statusBook: String? = ""
    private var ratingBook: Float = 0F
    private var startDate: String? = ""
    private var endDate: String? = ""

    private lateinit var ibMoveReading: ImageButton
    private lateinit var ibMoveRead: ImageButton
    private lateinit var ibMovePending: ImageButton
    private lateinit var ibMoveWished: ImageButton

    private lateinit var rvQuotes: RecyclerView
    private lateinit var adapter: QuotesRecyclerViewAdapter
    private var quoteListStrings = ArrayList<String>()
    private lateinit var tvNumQuotes: TextView

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        //Bring the data from another fragment
        parentFragmentManager.setFragmentResultListener("key", this,
            { key, bundle ->
                title = bundle.getString("titleKey")
                author = bundle.getString("authorKey")
                image = bundle.getString("imageKey")
                statusBook = bundle.getString("statusKey")

                tbMyBookDetails.title = title
                tvAuthorMyBookDetails.text = author

                if (image == null) {
                    Picasso.get()
                        .load(R.drawable.cover_not_avalaible)
                        .into(ivMyBookDetails)
                } else {
                    Picasso.get()
                        .load(image)
                        .into(ivMyBookDetails)
                }

                //ImageButton without opacity for status book
                when (statusBook) {
                    "Reading" -> ibMoveReading.alpha = 1F
                    "Read" -> ibMoveRead.alpha = 1F
                    "Pending" -> ibMovePending.alpha = 1F
                    "Wished" -> ibMoveWished.alpha = 1F
                }
                //ibMoveReading.setColorFilter(Color.LTGRAY, PorterDuff.Mode.SRC_IN)
                //ibMoveReading.colorFilter = null

                //Get date Start of firebase for this book
                firebaseRepo.getBookStartDate(statusBook!!, title, tvDateStartBook)
                //Get date End of firebase for this book
                firebaseRepo.getBookEndDate(statusBook!!, title, tvDateEndBook)
                //Get rating of firebase for this book
                firebaseRepo.getRatingBook(statusBook!!, title, rbMyBooksDetails)
                //Get all quotes of firebase for this book
                getQuotes(statusBook!!, title)
                //Get number of quotes in each book
                firebaseRepo.numberQuotes(statusBook!!, title, tvNumQuotes)

            })

    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {

        // Inflate the layout for this fragment
        val viewMyBooksDetailsFragment = inflater.inflate(R.layout.fragment_my_book_details, container, false)

        //Hide navigation menu
        val mainAcitivty = activity as AppCompatActivity?

        val btNavigationBar = mainAcitivty?.findViewById<BottomNavigationView>(R.id.bn_navigationView)
        btNavigationBar?.visibility = View.GONE

        tvAuthorMyBookDetails = viewMyBooksDetailsFragment.findViewById(R.id.tv_authorMyBookDetails)
        ivMyBookDetails = viewMyBooksDetailsFragment.findViewById(R.id.iv_myBookDetails)
        tvDateStartBook = viewMyBooksDetailsFragment.findViewById(R.id.tv_dateStartBook)
        tvDateEndBook = viewMyBooksDetailsFragment.findViewById(R.id.tv_dateEndBook)
        tvNumQuotes = viewMyBooksDetailsFragment.findViewById(R.id.tv_numQuotes)

        //custom toolbar
        tbMyBookDetails = viewMyBooksDetailsFragment.findViewById(R.id.tb_myBookDetails)
        tbMyBookDetails.inflateMenu(R.menu.menu_my_book_details)

        tbMyBookDetails.setNavigationIcon(R.drawable.ic_toolbar_arrow_back_24)
        tbMyBookDetails.setNavigationOnClickListener {
            fragmentManager?.popBackStack()
            btNavigationBar?.visibility = View.VISIBLE
        }

        //Menu items
        tbMyBookDetails.setOnMenuItemClickListener {
            when (it.itemId) {

                //Bottonm Sheet
                R.id.it_bottomSheet -> {
                    //pass data from this fragment to bottomSheet fragment
                    startDate = tvDateStartBook.text.toString()
                    endDate = tvDateEndBook.text.toString()

                    val bottomSheetFragment = BottomsheetFragment()
                    val bundle = Bundle()
                    bundle.putString("book_title_key", title)
                    bundle.putString("book_image_key", image)
                    bundle.putString("book_author_key", author)
                    bundle.putString("book_status_key", statusBook)
                    bundle.putString("book_start_date_key", startDate)
                    bundle.putString("book_end_date_key", endDate)
                    bundle.putFloat("book_rating_key", ratingBook)
                    bundle.putStringArrayList("book_quote_key", quoteListStrings)

                    parentFragmentManager.setFragmentResult("book_key", bundle)
                    bottomSheetFragment.show(parentFragmentManager, "BotoomSheetDialog")

                    true
                }

                else -> {
                    super.onOptionsItemSelected(it)
                }
            }
        }

        //Change color Collapsing toolbar
        val ctbMyBoookDetails = viewMyBooksDetailsFragment
            .findViewById<CollapsingToolbarLayout>(R.id.ctb_myBooksDetails)

        ctbMyBoookDetails.setExpandedTitleColor(Color.parseColor("#ffffff"))
        ctbMyBoookDetails.setCollapsedTitleTextColor(Color.parseColor("#ffffff"))

        //Calendar
        //ImageButton click to show DataPicker
        val ibPickInitDate = viewMyBooksDetailsFragment.findViewById<Button>(R.id.ib_pickInitDate)
        val ibPickEndDate = viewMyBooksDetailsFragment.findViewById<Button>(R.id.ib_pickEndDate)

        ibPickInitDate.setOnClickListener {
            datePicker(tvDateStartBook)
        }

        ibPickEndDate.setOnClickListener {
            datePicker(tvDateEndBook)
        }


        //RATING BAR
        rbMyBooksDetails = viewMyBooksDetailsFragment.findViewById<RatingBar>(R.id.rb_myBookDetails)

        rbMyBooksDetails.setOnRatingBarChangeListener { ratingBar, rating, fromUser ->
            /* Snackbar.make(viewMyBooksDetailsFragment, "Libro valorado con " +
                     "${if (rating % 1 == 0F){
                         rating.toInt()
                     }else{
                         rating
                     }
                     } estrellas sobre 5", Snackbar.LENGTH_SHORT).show()*/

            ratingBook = rating

            firebaseRepo.updtateRatingBook(statusBook!!, title, rating)
        }

        //CHANGE STATUS BOOK
        ibMoveReading = viewMyBooksDetailsFragment.findViewById<ImageButton>(R.id.ib_moveReading)
        ibMoveRead = viewMyBooksDetailsFragment.findViewById<ImageButton>(R.id.ib_moveRead)
        ibMovePending = viewMyBooksDetailsFragment.findViewById<ImageButton>(R.id.ib_movePending)
        ibMoveWished = viewMyBooksDetailsFragment.findViewById<ImageButton>(R.id.ib_moveWished)
        ibMoveReading.alpha = 0.3F
        ibMoveRead.alpha = 0.3F
        ibMovePending.alpha = 0.3F
        ibMoveWished.alpha = 0.3F

        ibMoveReading.setOnClickListener {
            //ibMoveReading.setImageResource(R.drawable.ic_reading)
            ibMoveReading.alpha = 1F
            ibMoveRead.alpha = 0.3F
            ibMovePending.alpha = 0.3F
            ibMoveWished.alpha = 0.3F
            FirebaseRepo().deleteBook(title, statusBook!!)
            FirebaseRepo().setBookData(
                "Reading",
                title,
                author,
                image,
                tvDateStartBook.text.toString(),
                tvDateEndBook.text.toString(),
                ratingBook, quoteListStrings
            )
            Snackbar.make(
                viewMyBooksDetailsFragment,
                "Libro movido a leyendo",
                Snackbar.LENGTH_SHORT
            ).show()
        }

        ibMoveRead.setOnClickListener {
            ibMoveRead.alpha = 1F
            ibMoveReading.alpha = 0.3F
            ibMovePending.alpha = 0.3F
            ibMoveWished.alpha = 0.3F
            FirebaseRepo().deleteBook(title, statusBook!!)
            FirebaseRepo().setBookData(
                "Read",
                title,
                author,
                image,
                tvDateStartBook.text.toString(),
                tvDateEndBook.text.toString(),
                ratingBook, quoteListStrings
            )
            Snackbar.make(
                viewMyBooksDetailsFragment,
                "Libro movido a leídos",
                Snackbar.LENGTH_SHORT
            ).show()
        }

        ibMovePending.setOnClickListener {
            //ibMoveRead.setImageResource(R.drawable.ic_read)
            //ibMoveReading.setImageResource(R.drawable.ic_reading)
            ibMovePending.alpha = 1F
            ibMoveReading.alpha = 0.3F
            ibMoveRead.alpha = 0.3F
            ibMoveWished.alpha = 0.3F
            FirebaseRepo().deleteBook(title, statusBook!!)
            FirebaseRepo().setBookData(
                "Pending",
                title,
                author,
                image,
                tvDateStartBook.text.toString(),
                tvDateEndBook.text.toString(),
                ratingBook, quoteListStrings
            )
            Snackbar.make(
                viewMyBooksDetailsFragment,
                "Libro movido a pendientes",
                Snackbar.LENGTH_SHORT
            ).show()
        }

        ibMoveWished.setOnClickListener {
            //ibMoveRead.setImageResource(R.drawable.ic_read)
            //ibMoveReading.setImageResource(R.drawable.ic_reading)
            ibMoveWished.alpha = 1F
            ibMovePending.alpha = 0.3F
            ibMoveReading.alpha = 0.3F
            ibMoveRead.alpha = 0.3F
            //FirebaseRepo().deleteBook(title, statusBook!!)
            FirebaseRepo().setBookData(
                "Wished",
                title,
                author,
                image,
                tvDateStartBook.text.toString(),
                tvDateEndBook.text.toString(),
                ratingBook, quoteListStrings
            )
            Snackbar.make(
                viewMyBooksDetailsFragment,
                "Libro añadido a lista de deseos",
                Snackbar.LENGTH_SHORT
            ).show()
        }


        //QUOTES RECYCLER VIEW
        rvQuotes = viewMyBooksDetailsFragment.findViewById<RecyclerView>(R.id.rv_quoutes)
        rvQuotes.setHasFixedSize(true)
        rvQuotes.layoutManager = LinearLayoutManager(context)

        setupRecyclerView(rvQuotes)

        //Add Quote
        val btAddQuote = viewMyBooksDetailsFragment.findViewById<Button>(R.id.bt_addQuote)
        btAddQuote.setOnClickListener {
            val builder = AlertDialog.Builder(context, R.style.AlertDialogTheme)
            val dialogLayout = LayoutInflater.from(context)
                .inflate(
                    R.layout.dialog_custom_quote,
                    viewMyBooksDetailsFragment.findViewById(R.id.cl_dialog_container)
                )

            val tvTitleDialogQuote = dialogLayout.findViewById<TextView>(R.id.tv_titleDialogQuote)
            val etmQuote = dialogLayout.findViewById<EditText>(R.id.etm_quote)
            val btSaveQuote = dialogLayout.findViewById<Button>(R.id.bt_saveQuote)

            tvTitleDialogQuote.text = title

            builder.setView(dialogLayout)
            val alertDialog = builder.create()
            if (alertDialog.window != null) {
                alertDialog.window!!.setBackgroundDrawable(ColorDrawable(0))
            }
            alertDialog.show()

            btSaveQuote.setOnClickListener {
                quoteListStrings.add("\"${etmQuote.text}\"")
                firebaseRepo.updtateBookQuotes(statusBook!!, title, "\"${etmQuote.text}\"")
                adapter.notifyDataSetChanged()
                firebaseRepo.numberQuotes(statusBook!!, title, tvNumQuotes)
                alertDialog.dismiss()
            }

        }

        return viewMyBooksDetailsFragment

    }

    private fun setupRecyclerView(recyclerView: RecyclerView) {
        adapter = QuotesRecyclerViewAdapter(quoteListStrings, object : ClickListenerQuote {
            override fun onClickMenu(view: View, index: Int) {
                showPopupMenu(index, view)
            }
        })
        recyclerView.adapter = adapter
    }


    private fun showPopupMenu(index: Int, view: View) {
        //val menuItemView = viewReadingFragment.findViewById<View>(R.id.ib_menuCardview)
        val popupMenu = PopupMenu(view.context, view)

        //For we can put icons with text
        try {
            val fields: Array<Field> = popupMenu.javaClass.declaredFields
            for (field in fields) {
                if ("mPopup" == field.name) {
                    field.isAccessible = true
                    val menuPopupHelper: Any = field[popupMenu]!!
                    val classPopupHelper = Class.forName(menuPopupHelper.javaClass.name)
                    val setForceIcons: Method = classPopupHelper.getMethod(
                        "setForceShowIcon",
                        Boolean::class.javaPrimitiveType
                    )
                    setForceIcons.invoke(menuPopupHelper, true)
                    break
                }
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }

        popupMenu.menuInflater.inflate(R.menu.context_menu_quote, popupMenu.menu)
        popupMenu.setOnMenuItemClickListener { item ->
            when (item.itemId) {
                R.id.it_deleteQuote -> {
                    //Delete quote
                    FirebaseRepo().deleteQuote(quoteListStrings[index], title!!, statusBook!!)
                    //Update list quotes
                    getQuotes(statusBook!!, title)
                    //Update number of quotes
                    firebaseRepo.numberQuotes(statusBook!!, title, tvNumQuotes)
                    // Snackbar.make(viewM, "Libro eliminado", Snackbar.LENGTH_SHORT).show()
                }

                else -> {
                }
            }
            true
        }
        //popupMenu.setForceShowIcon(true)

        popupMenu.show()

    }

    fun getQuotes(status: String, title: String?) {
        val docRef = db.collection("Users").document(mAuth.currentUser?.email.toString())
            .collection(status).document(title!!)
        docRef.get()
            .addOnCompleteListener { task: Task<DocumentSnapshot?> ->
                if (task.isSuccessful) {
                    val document = task.result
                    if (document?.get("quotes") != null) {
                        quoteListStrings = (document["quotes"] as ArrayList<String>)
                        //Log.d(TAG, quoteListStrings[1].toString())
                        adapter.setItems(quoteListStrings)
                    }
                }
            }

            .addOnFailureListener { exception ->
                Log.d(TAG, "get failed with ", exception)
            }


    }

    fun datePicker(textView: TextView) {
        //CALENDAR
        val c = Calendar.getInstance()
        val year = c.get(Calendar.YEAR)
        val month = c.get(Calendar.MONTH)
        val day = c.get(Calendar.DAY_OF_MONTH)

        val dpd =
            DatePickerDialog(
                this.requireContext(),
                DatePickerDialog.OnDateSetListener { view, mYear, mMonth, mDay ->
                    textView.text = "" + mDay + "/" + (mMonth + 1).toString() + "/" + mYear

                    val firebaseRepo = FirebaseRepo()

                    if (textView == tvDateStartBook)
                        firebaseRepo.updtateBookDate(
                            statusBook!!,
                            title!!,
                            textView.text.toString()
                        )
                    else {
                        firebaseRepo.updtateBookEndDate(
                            statusBook!!,
                            title!!,
                            textView.text.toString()
                        )
                    }

                },
                year,
                month,
                day
            )
        dpd.show()
    }

}