package com.villanueva.production.livooks.fragments

import android.app.Activity
import android.content.Context
import android.net.Uri
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.inputmethod.InputMethodManager
import android.widget.ProgressBar
import android.widget.SearchView
import androidx.appcompat.app.AppCompatActivity
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.android.volley.Request
import com.android.volley.toolbox.StringRequest
import com.android.volley.toolbox.Volley
import com.google.android.material.snackbar.Snackbar
import com.google.gson.Gson
import com.villanueva.production.livooks.R
import com.villanueva.production.livooks.adapters.SearchRecyclerViewAdapter
import com.villanueva.production.livooks.interfaces.ClickListenerSearch
import com.villanueva.production.livooks.model.modeAPI.BookContent

class SearchFragment : Fragment() {

    //APIKEY Google Books API
    private val APIKEY = "AIzaSyAKUqovXOosO_PYm5KA19rX3JmZJWP754c"
    private lateinit var rvListaBooks: RecyclerView
    private var adapter: SearchRecyclerViewAdapter? = null

    private lateinit var pbSearch: ProgressBar
    private lateinit var viewSearchFragment:View

    private var codeScanner: String? =null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        codeScanner = arguments?.getString("key_codeScanner")
        if(codeScanner!=null){
            fetchBook(codeScanner!!)
            codeScanner =null
        }

    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        // Inflate the layout for this fragment
        viewSearchFragment = inflater.inflate(R.layout.fragment_search, container, false)

        val mainActivity = activity as AppCompatActivity?
        val searchView = mainActivity?.findViewById<SearchView>(R.id.sv_searchviewTrue)

        //Recycler view
        rvListaBooks = viewSearchFragment.findViewById(R.id.rv_listBooks)
        rvListaBooks.setHasFixedSize(true)
        rvListaBooks.layoutManager = LinearLayoutManager(context)

        //Implement progress bar while charge data from database Firestore
        pbSearch = viewSearchFragment.findViewById(R.id.pb_search)

        if (!searchView?.query.isNullOrEmpty()) {
            fetchBook(searchView?.query.toString())
        }

        //Configure searchview for introduce text and start search when submit it
        val queryTextListener: SearchView.OnQueryTextListener =
            object : SearchView.OnQueryTextListener {
                override fun onQueryTextSubmit(queryString: String): Boolean {
                    pbSearch.visibility = View.VISIBLE
                    //Start the search in the API Google Books
                    fetchBook(queryString)
                    //Call function for hide keyboard
                    hideKeyboard()
                    searchView?.clearFocus()
                    return true
                }

                override fun onQueryTextChange(queryString: String): Boolean {
                    //Automatic search when write more than 5 characters in the searchView
                   if(queryString.length>5){
                        fetchBook(queryString)
                    }
                    return false
                }
            }
        searchView?.setOnQueryTextListener(queryTextListener)

        return viewSearchFragment
    }



    //Function for hide keyboard when press keyboard search
    fun Fragment.hideKeyboard() {
        view?.let { activity?.hideKeyboard(it) }
    }
    fun Context.hideKeyboard(view: View) {
        val inputMethodManager =
            getSystemService(Activity.INPUT_METHOD_SERVICE) as InputMethodManager
        inputMethodManager.hideSoftInputFromWindow(view.windowToken, 0)
    }

    //Función for search the book in the API and show listed in recycler view
    private fun fetchBook(queryString: String) {

        // Instantiate the RequestQueue volley library
        val queue = Volley.newRequestQueue(this.context)

        //Configure API Google Books
        val BOOK_BASE_URL = "https://www.googleapis.com/books/v1/volumes?q=$queryString"
        val PRINT_TYPE = "printType"
        val LANG_RESTRICT = "langRestrict"
        val MAX_RESULT = "maxResults"
        val PARAM_API_KEY = "key"

        val builtUri = Uri.parse(BOOK_BASE_URL).buildUpon()
            .appendQueryParameter(MAX_RESULT, "10")
            .appendQueryParameter(PRINT_TYPE, "books")
            .appendQueryParameter(LANG_RESTRICT, "es")
            .appendQueryParameter(PARAM_API_KEY, APIKEY)
            .build()
        val url = builtUri.toString()



        // Request a string response from the provided URL.
        val stringRequest = StringRequest(Request.Method.GET, url, { response ->
            val gson = Gson()
            val books = gson.fromJson(response, BookContent::class.java)

            adapter = SearchRecyclerViewAdapter(books.items, object : ClickListenerSearch {
                override fun onClickSearch(view: View, index: Int) {

                    //Go to details of book choosed in the search list
                    val transaction = activity?.supportFragmentManager?.beginTransaction()
                    transaction?.replace(
                        R.id.fc_fragmentContainer, BookSearchDetailsFragment(),
                        "main"
                    )?.addToBackStack("main")
                    transaction?.commit()

                    //Pass data from one fragment to another
                    val bundle = Bundle()
                    bundle.putString("titleKey", books.items?.get(index)?.volumeInfo?.title)
                    bundle.putStringArrayList(
                        "authorKey",
                        books.items?.get(index)?.volumeInfo?.authors
                    )
                    bundle.putString(
                        "imageKey",
                        books.items?.get(index)?.volumeInfo?.imageLinks?.thumbnail
                    )
                    bundle.putString(
                        "descriptionKey",
                        books.items?.get(index)?.volumeInfo?.description
                    )
                    bundle.putFloat(
                        "ratingKey",
                        books.items?.get(index)?.volumeInfo!!.averageRating
                    )
                    bundle.putStringArrayList(
                        "categoriesKey",
                        books.items[index].volumeInfo?.categories
                    )
                    parentFragmentManager.setFragmentResult("key", bundle)


                }

                override fun onClickMenu(view: View, index: Int) {
                }
            })

            rvListaBooks.adapter = adapter
            pbSearch.visibility = View.GONE
        },
            {
                Snackbar.make(
                    viewSearchFragment, "Fallo al conectar con el servidor, revise su" +
                            " conexión a internet y vuelva a intentarlo", Snackbar.LENGTH_LONG
                ).show()
                pbSearch.visibility = View.GONE
            })
        // Add the request to the RequestQueue.
        queue.add(stringRequest)
    }
}


